#!/usr/bin/python
# -*- coding:utf-8 -*-

import os, string, re, sys, types

#from LocationData import *
from dom import *

prefectureTable = { "北海道":0, "青森県":1, "岩手県":2, "宮城県":3,
                    "秋田県":4, "山形県":5, "福島県":6, "茨城県":7,
                    "栃木県":8, "群馬県":9, "埼玉県":10, "千葉県":11,
                    "東京都":12, "神奈川県":13, "新潟県":14, "富山県":15,
                    "石川県":16, "福井県":17, "山梨県":18, "長野県":19,
                    "岐阜県":20, "静岡県":21, "愛知県":22, "三重県":23,
                    "滋賀県":24, "京都府":25, "大阪府":26, "兵庫県":27,
                    "奈良県":28, "和歌山県":29, "鳥取県":30, "島根県":31,
                    "岡山県":32, "広島県":33, "山口県":34, "徳島県":35,
                    "香川県":36, "愛媛県":37, "高知県":38, "福岡県":39,
                    "佐賀県":40, "長崎県":41, "熊本県":42, "大分県":43,
                    "宮崎県":44, "鹿児島県":45, "沖縄県":46, };

class Location:
    def __init__( self, area, locations ):
        self.area = area
        if type( locations ) == types.StringType:
            self.locations = re.split( ',', locations )
        elif type( locations ) == types.ListType or type( locations ) == types.TupleType:
            self.locations = locations
        else:
            raise Exception, "cannot resolve locations"
            pass
        pass

    def get_name( self ):
        return self.area

    def get_location_name( self, num ):
        if len( self.locations ) > num: return self.locations[ num ]
        return None
    
    def get_location_list( self ):
        if len( self.locations ) == 0: return "()"
        output = "("
        for l in self.locations:
            output += "\"%s\"," % ( l )
            pass
        output += ")"
        return output

    def contains( self, location ):
        for l in self.locations:
            if l == location: return True
            pass
        return False

    def find( self, location ):
        for i in range( 0, len( self.locations ) ):
            if self.locations[ i ] == location: return i
            pass
        return -1

    def __repr__( self ):
        output = self.area
        if len( self.locations ) == 0: return output
        output += " {" + self.locations[ 0 ]
        for l in self.locations[ 1: ]:
            output += "," + l
            pass
        output += "}"
        return output
    pass

class Area:
    def __init__( self, prefname, prefcode = None ):
        if prefcode == None:
            prefcode = prefectureTable[ prefname ]
            pass
        self.name = prefname
        self.pref = prefcode
        self.regions = []
        pass

    def get_prefecture_code( self ):
        return self.pref

    def get_prefecture_name( self ):
        return self.name

    def get_regions( self ):
        return self.regions

    def add_location( self, location ):
        self.regions.append( location )
        pass

    def resolve_region( self, location ):
        for i in range( 0, len( self.regions ) ):
            if self.regions[ i ].contains( location ):
                return i
            pass
        return -1

    def convert_code_to_name( self, code ):
        rnum = string.atoi( code[ 2: 5 ] )
        lnum = string.atoi( code[ 5: 8 ] )
        if len( self.regions ) > rnum:
            lname = self.regions[ rnum ].get_location_name( lnum )
            if lname:
                return self.regions[ rnum ].get_name() + " " + lname
            pass
        return None
    
    def resolve_code( self, location ):
        for i in range( 0, len( self.regions ) ):
            n = self.regions[ i ].find( location )
            if n >= 0:
                #if self.regions[ i ].contains( location ):
                return ("00%d" % i)[ -3: ] + ("000%d" % n)[ -3: ] 
            pass
        return None
        

    def __repr__( self ):
        output  = "PREF NAME :%s\n" % self.name
        output += "PREF CODE :%d\n" % self.pref
        output += "REGIOINS  :%d\n" % ( len( self.regions ) )
        for r in self.regions: output += "%s\n" % repr( r )
        return output
    pass

class MonitoringPoints:
    def __init__( self ):
        self.points = {}
        for p in prefectureTable.keys():
            n = prefectureTable[ p ]
            self.points[ n ] = Area( p, n )
            pass
        pass

    def resolve_code( self, pref, location ):
        if type( pref ) == types.StringType:
            pref = prefectureTable[ pref ]
            pass
        if type( pref ) != types.IntType:
            raise Exception, "no prefecture " + repr( pref )
        acode = self.points[ pref ].resolve_code( location )
        if acode == None: return None
        code = ("000%d" % pref)[ -2: ]
        return code + acode

    def convert_code_to_name( self, code ):
        pnum = string.atoi( code[ 0: 2 ] )
        point = self.points[ pnum ]
        pname = point.get_prefecture_name()
        aname = point.convert_code_to_name( code )
        if pname == None or aname == None: return None
        return pname + " " + aname
        #for p in self.points.keys():
        #    if p.get_prefecture_code() == pnum:
        #return point.get_prefecture_name() + " " + point.convert_code_to_name( code )

    def add_location( self, pref, area, locations ):
        if type( pref ) == types.IntType:
            self.points[ pref ].add_location( Location( area, locations ) )
        elif type( pref ) == types.StringType and prefectureTable.has_key( pref ):
            self.points[ prefectureTable[ pref ] ].add_location( Location( area, locations ) )
            pass
        else:
            raise Exception, "no prefecture " + repr( pref )
        pass

    def generate_definition_command( self, varname = "monitoringPoints" ):
        command = "%s = MonitoringPoints()\n" % varname
        for point in self.points.keys():
            for r in self.points[ point ].get_regions():
                command += "%s.add_location( %d, \"%s\", %s )\n" \
                           % ( varname, point, r.get_name(),
                               r.get_location_list() )
                pass
            pass
        return command

    def __repr__( self ):
        output = ""
        for p in self.points.keys():
            output += "%s\n" % repr( self.points[ p ] )
            pass
        return output
    pass


if __name__ == "__main__":
    monitorPoints = MonitoringPoints()
    filename = sys.argv[ 1 ]
    p = DOMConverter()
    for line in open( filename ).readlines():
        p.feed( line.strip() )
        pass
    doc = p.get_document()
    #
    re_paren = re.compile( u'(^|、)[^、]*(（([^）]+)）)' )
    elem = doc.get_element_at_path( "/html/body/div,2/table/tbody/tr,2" )
    if elem:
        ( pref, area, locations ) = elem.get_text( "--*--" ).split( "--*--" )
        pass
    #data = None
    while elem != None:
        items = elem.get_text( "--*--" ).split( "--*--" )
        flag_valid = True
        if len( items ) == 1:
            locations = items[ 0 ]
            #flag_valid = False
        elif len( items ) == 2:
            area, locations = items[ 0: 2 ]
        elif len( items ) >= 3:
            pref, area, locations = items[ 0: 3 ]
        else:
            flag_valid = False
            pass
        if flag_valid:
            uloc = unicode( locations, 'utf-8' )
            m = re_paren.search( uloc )
            while m:
                uloc = uloc[ 0: m.start() ] + uloc[ m.end(): ]
                uloc += u'、' + m.group( 3 )
                m = re_paren.search( uloc )
                pass
            locs = uloc.split( u'、' )
            locations = ''
            flag_first = True
            for l in locs[ 0: ]:
                if len( l ) > 0:
                    if flag_first:
                        locations += l.encode( 'utf-8' ).strip()
                        flag_first = False
                    else:
                        locations += "," + l.encode( 'utf-8' ).strip()
                        pass
                    pass
                pass

            monitorPoints.add_location( pref, area, locations )
            pass
        try:
            elem = elem.next_sibling()
        except:
            break
        pass

    print monitorPoints.generate_definition_command()
    #print monitorPoints.resolve_code( "鹿児島県", "大口市" )
    pass
